# -*- coding: utf-8 -*-
from datetime import datetime
from .channel import Channel
from copy import deepcopy

import json, time, base64

def loggedIn(func):
    def checkLogin(*args, **kwargs):
        if args[0].isLogin:
            return func(*args, **kwargs)
        else:
            args[0].callback.default('You want to call the function, you must login to LINE')
    return checkLogin
    
class Timeline(Channel):

    def __init__(self):
        if not self.channelId:
            self.channelId = self.server.CHANNEL_ID['LINE_TIMELINE']
        Channel.__init__(self, self.channel, self.channelId, False)
        self.tl = self.getChannelResult()
        self.__loginTimeline()
        
    def __loginTimeline(self):
        self.server.setTimelineHeadersWithDict({
            'Content-Type': 'application/json; charset=UTF-8',
            'User-Agent': 'androidapp.line/11.5.1 (Linux; U; Android 7.0; en-GB; Redmi Note 4 Build/NRD90M)',
            'X-Line-Mid': self.profile.mid,
            'X-Line-Carrier': self.server.CARRIER,
            'X-LSR': 'ID',
            'X-LPV': '1',
            'X-Line-Application': 'ANDROID 11.5.1 Android OS 7.0',
            'Accept-Encoding': 'gzip',
            'X-Line-ChannelToken': self.tl.channelAccessToken
        })
        self.LineLegyDomain = "https://legy-jp-addr.line.naver.jp"
        self.LineHostDomain = "https://ga2.line.naver.jp"
        self.LineGwzDomain  = "https://gwz.line.naver.jp"
        self.LineObsDomain  = "https://obs-sg.line-apps.com"
        self.LineLiffDomain = "https://api.line.me/message/v3/share"
        self.LinePermission = "https://access.line.me/dialog/api/permissions"
        self.profileDetail = self.getProfileDetail()

    """Timeline"""

    @loggedIn
    def getFeed(self, postLimit=10, commentLimit=1, likeLimit=1, order='TIME'):
        params = {'postLimit': postLimit, 'commentLimit': commentLimit, 'likeLimit': likeLimit, 'order': order}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/feed/list.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def getHomeProfile(self, mid=None, postLimit=10, commentLimit=1, likeLimit=1):
        if mid is None:
            mid = self.profile.mid
        params = {'homeId': mid, 'postLimit': postLimit, 'commentLimit': commentLimit, 'likeLimit': likeLimit, 'sourceType': 'LINE_PROFILE_COVER'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/post/list.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def getProfileDetail(self, mid=None, styleMediaVersion='v2', storyVersion='v6'):
        if mid is None:
            mid = self.profile.mid
        params = {
            'homeId': mid,
            'styleMediaVersion': styleMediaVersion,
            'storyVersion': storyVersion
        }
        hr = self.server.additionalHeaders(self.server.timelineHeaders, {
            'x-lhm': "GET",
            'x-line-global-config': "discover.enable=true; follow.enable=true",
        })
        url = self.server.urlEncode(self.LineHostDomain+'/hm', '/api/v1/home/profile.json', params)
        r = self.server.postContent(url, headers=hr, data='')
        return r.json()
        
    @loggedIn
    def updateProfileCoverById(self, objId):
        params = {'coverImageId': objId}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/home/updateCover.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def updateProfileCoverByIdV2(self,mid=None,objid=None,vObjid=None):
        data = {
            "homeId": mid if mid else self.profile.mid,
            "coverObjectId": objid,
            "storyShare": False,
            "meta":{} # heh
        }
        if vObjid:
            data['videoCoverObjectId'] = vObjid
        hr = self.server.additionalHeaders(deepcopy(self.server.timelineHeaders), {
            'x-lhm': "POST",
            'Content-type': "application/json",
        })
        r = self.server.postContent("https://ga2.line.naver.jp/hm/api/v1/home/cover.json",headers=hr, data=json.dumps(data))
        return r.json()
        
    @loggedIn
    def getProfileCoverId(self, mid=None):
        if mid is None:
            mid = self.profile.mid
        home = self.getProfileDetail(mid)
        return home['result']['objectId']

    @loggedIn
    def getProfileCoverURL(self, mid=None):
        if mid is None:
            mid = self.profile.mid
        profileDetail = self.getProfileDetail(mid)
        if 'videoCoverObsInfo' in profileDetail['result']:
            params = {'userid': mid, 'oid': profileDetail['result']['videoCoverObsInfo']['objectId']}
            url = self.server.urlEncode(self.LineObsDomain, '/myhome/vc/download.nhn', params)
            return url
        else:
            params = {'userid': mid, 'oid': profileDetail['result']['coverObsInfo']['objectId']}
            url = self.server.urlEncode(self.LineObsDomain, '/myhome/c/download.nhn', params)
            return url

    """Post"""

    @loggedIn
    def createPost(self, text, holdingTime=None):
        params = {'homeId': self.profile.mid, 'sourceType': 'TIMELINE'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/post/create.json', params)
        payload = {'postInfo': {'readPermission': {'type': 'ALL'}}, 'sourceType': 'TIMELINE', 'contents': {'text': text}}
        if holdingTime != None:
            payload["postInfo"]["holdingTime"] = holdingTime
        data = json.dumps(payload)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def sendPostToTalk(self, mid, postId):
        if mid is None:
            mid = self.profile.mid
        params = {'receiveMid': mid, 'postId': postId}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/post/sendPostToTalk.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def createComment(self, mid, contentId, text, contentMetadata={}):
        if mid is None:
            mid = self.profile.mid
        params = {'homeId': mid, 'sourceType': 'TIMELINE'}
        data = {
            'contentId': contentId,
        }
        if '@!' in text:
            text = text.replace('@!','@')
        if '@' in text:
            data.update({
                'actorId': mid,
                'commentText': str(text),
                'recallInfos':[
                    {
                        'start': text.index('@'),
                        'end': text.index('@')+1,
                        'user':{
                            'actorId': mid
                        }
                    }
                ]
            })
        else:
            data['commentText'] = str(text)
        if contentMetadata != {}:
            data['secret'] = False
            data['contentsList'] = []
            data['contentsList'].append({
                'categoryId' : 'sticker',
                'extData' : {
                    'id' : contentMetadata["STKID"],
                    'packageId' : contentMetadata["STKPKGID"],
                    'packageVersion' : 1
                }
            })
        url = self.server.urlEncode(self.LineGwzDomain, '/mh/api/v52/comment/create.json', params)
        data = json.dumps(data)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def deleteComment(self, mid, postId, commentId):
        if mid is None:
            mid = self.profile.mid
        params = {'homeId': mid, 'sourceType': 'TIMELINE'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/comment/delete.json', params)
        data = {'commentId': commentId, 'activityExternalId': postId, 'actorId': mid}
        data = json.dumps(data)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def likePost(self, mid, postId, likeType=1001):
        if mid is None:
            mid = self.profile.mid
        if likeType not in [1001,1002,1003,1004,1005,1006]:
            raise Exception('Invalid parameter likeType')
        params = {'homeId': mid, 'sourceType': 'TIMELINE'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/like/create.json', params)
        data = {'likeType': likeType, 'activityExternalId': postId, 'actorId': mid}
        data = json.dumps(data)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def unlikePost(self, mid, postId):
        if mid is None:
            mid = self.profile.mid
        params = {'homeId': mid, 'sourceType': 'TIMELINE'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/like/cancel.json', params)
        data = {'activityExternalId': postId, 'actorId': mid}
        data = json.dumps(data)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def getStory(self, mid=None):
        if mid == None:
            mid = self.profile.mid
        params = {"userMid": mid}
        url = self.server.urlEncode('https://ga2.line.naver.jp', '/st/api/v6/story')
        result = self.server.postContent(url, data=json.dumps(params), headers=self.server.timelineHeaders).json()
        return result

    @loggedIn
    def getRecentStoryV2(self):
        hr = self.server.additionalHeaders(self.server.timelineHeaders, {
            'x-lhm': 'POST',
            'content-type': "application/json"
        })
        r = self.server.postContent(self.LineHostDomain + '/st/api/v6/story', data=data, headers=hr)
        return r.json()

    @loggedIn
    def getStoryMedia(self, to, mid):
        story = self.getStory(mid)
        listnya = []
        if story['message'] == 'success':
            if story['result']['contents']:
                for content in story['result']['contents']:
                    if content['media'][0]['mediaType'] == 'IMAGE':
                        path = 'https://obs.line-scdn.net/' + content['media'][0]['hash']
                        listnya.append({"type":"IMAGE", "url":path})
                    if content['media'][0]['mediaType'] == 'VIDEO':
                        path = 'https://obs.line-scdn.net/' + content['media'][0]['hash']
                        listnya.append({"type":"VIDEO", "url":path})                        
        print(listnya)
        return listnya
    """Group Post"""
    @loggedIn
    def createPostGroup(self, text,to, holdingTime=None,textMeta=[]):
        params = {'homeId': to, 'sourceType': 'GROUPHOME'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v39/post/create.json', params)
        payload = {'postInfo': {'readPermission': {'type': 'ALL'}}, 'sourceType': 'GROUPHOME', 'contents': {'text': text,'textMeta':textMeta}}
        if holdingTime != None:
            payload["postInfo"]["holdingTime"] = holdingTime
        data = json.dumps(payload)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def createPostGroupR(self, text,to):
        params = {'homeId': to, 'sourceType': 'GROUPHOME'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v39/relay/create.json', params)
        payload = {'postInfo': {'readPermission': {'type': 'ALL'}}, 'sourceType': 'GROUPHOME', 'contents': {'text': text,'textMeta':textMeta}}
        if holdingTime != None:
            payload["postInfo"]["holdingTime"] = holdingTime
        data = json.dumps(payload)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def createGroupPost(self, mid, text):
        payload = {'postInfo': {'readPermission': {'homeId': mid}}, 'sourceType': 'TIMELINE', 'contents': {'text': text}}
        data = json.dumps(payload)
        r = self.server.postContent(self.server.LINE_TIMELINE_API + '/v45/post/create.json', data=data, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def createGroupAlbum(self, mid, name):
        data = json.dumps({'title': name, 'type': 'image'})
        params = {'homeId': mid,'count': '1','auto': '0'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_MH, '/album/v3/album.json', params)
        r = self.server.postContent(url, data=data, headers=self.server.timelineHeaders)
        if r.status_code != 201:
            raise Exception('Create a new album failure.')
        return True

    @loggedIn
    def deleteGroupAlbum(self, mid, albumId):
        params = {'homeId': mid}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_MH, '/album/v3/album/%s' % albumId, params)
        r = self.server.deleteContent(url, headers=self.server.timelineHeaders)
        if r.status_code != 201:
            raise Exception('Delete album failure.')
        return True
    
    @loggedIn
    def getGroupPost(self, mid, postLimit=10, commentLimit=1, likeLimit=1):
        params = {'homeId': mid, 'commentLimit': commentLimit, 'likeLimit': likeLimit, 'sourceType': 'TALKROOM'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_API, '/v45/post/list.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    """Group Album"""

    @loggedIn
    def getGroupAlbum(self, mid):
        params = {'homeId': mid, 'type': 'g', 'sourceType': 'TALKROOM'}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_MH, '/album/v3/albums.json', params)
        r = self.server.getContent(url, headers=self.server.timelineHeaders)
        return r.json()

    @loggedIn
    def changeGroupAlbumName(self, mid, albumId, name):
        data = json.dumps({'title': name})
        params = {'homeId': mid}
        url = self.server.urlEncode(self.server.LINE_TIMELINE_MH, '/album/v3/album/%s' % albumId, params)
        r = self.server.putContent(url, data=data, headers=self.server.timelineHeaders)
        if r.status_code != 201:
            raise Exception('Change album name failure.')
        return True

    @loggedIn
    def addImageToAlbum(self, mid, albumId, path):
        file = open(path, 'rb').read()
        params = {
            'oid': int(time.time()),
            'quality': '90',
            'range': len(file),
            'type': 'image'
        }
        hr = self.server.additionalHeaders(self.server.timelineHeaders, {
            'Content-Type': 'image/jpeg',
            'X-Line-Mid': mid,
            'X-Line-Album': albumId,
            'x-obs-params': self.genOBSParams(params,'b64')
        })
        r = self.server.getContent(self.server.LINE_OBS_DOMAIN + '/album/a/upload.nhn', data=file, headers=hr)
        if r.status_code != 201:
            raise Exception('Add image to album failure.')
        return r.json()

    @loggedIn
    def getImageGroupAlbum(self, mid, albumId, objId, returnAs='path', saveAs=''):
        if saveAs == '':
            saveAs = self.genTempFile('path')
        if returnAs not in ['path','bool','bin']:
            raise Exception('Invalid returnAs value')
        hr = self.server.additionalHeaders(self.server.timelineHeaders, {
            'Content-Type': 'image/jpeg',
            'X-Line-Mid': mid,
            'X-Line-Album': albumId
        })
        params = {'ver': '1.0', 'oid': objId}
        url = self.server.urlEncode(self.server.LINE_OBS_DOMAIN, '/album/a/download.nhn', params)
        r = self.server.getContent(url, headers=hr)
        if r.status_code == 200:
            self.saveFile(saveAs, r.raw)
            if returnAs == 'path':
                return saveAs
            elif returnAs == 'bool':
                return True
            elif returnAs == 'bin':
                return r.raw
        else:
            raise Exception('Download image album failure.')
            
    @loggedIn
    def likeStory(self, contentId, likeType: int):
        liked = [1001, 1002, 1003, 1004, 1005, 1006]
        if likeType not in liked:
            raise Exception ('Whats type like huh?')
        params = {"tsId": "", "contentId": contentId, "like": True, "likeType": str(likeType)}
        return self.server.postContent(self.LineHostDomain + '/st/api/v6/story/content/like', data=json.dumps(params), headers=self.server.timelineHeaders).json()

    @loggedIn
    def commentStory(self, mid=None, contentId=None, text=None):
        if not isinstance(mid and contentId, str):
            raise Exception('mid and contentId is required.')
        if text == None:
            text = 'Auto comment by DeadLine™'
        params = {"to":{"userMid": mid, "friendType": "","tsId": ""}, "contentId": contentId, "message": text}
        return self.server.postContent(self.LineHostDomain + '/st/api/v6/story/message/send', data=json.dumps(params), headers=self.server.timelineHeaders).json()

    @loggedIn
    def getRecentStory(self):
        params = {"lastRequestTime": int(time.time()), "lastTimelineVisitTime": int(time.time())}
        return self.server.postContent(self.LineHostDomain + '/st/api/v6/story/recentstory/list', headers = self.server.timelineHeaders, data = json.dumps(params)).json()

    @loggedIn
    def updateStory(self, objId=None, obsHash=None, mediaType='image'):
        if not isinstance(objId and obsHash, str):
            raise Exception('You have to upload and get the x-obs-oid & x-obs-hash')
        params = {"content": {"sourceType": "USER", "contentType": "USER", "media":[{"oid": objId, "service": "story", "sid": "st", "hash": obsHash, "mediaType": mediaType.upper()}]},"shareInfo":{"shareType": "FRIEND"}}
        self.server.timelineHeaders.update({'X-Line-BDBTemplateVersion': 'v1'})
        result = self.server.postContent('https://ga2s.line.naver.jp/st/api/v6/story/content/create', headers = self.server.timelineHeaders, data=json.dumps(params)).json()
        print(result)
        return result
